/******************************************************************************
Author:          David Szybala
Website:        www.davidszybala.com
Alt Website:   www.twistedgerbil.com
Program:		show_bounds.ms
Date:			    11/10/10
Description:	This script will create helpers to display a selection bounding box.
******************************************************************************/

try (destroyDialog show_bounds_rollout) catch()


struct ShowBounds
(
    point_helpers = #(),
    
    /**************************************************************************
    Returns a point helper with the default settings
    **************************************************************************/
    fn get_point_helper =
    (
        local point_helper = point centermarker:false axistripod:false cross:true box:false
        
        point_helper --return value
    ), --end fn
    
    /**************************************************************************
    Creates and stores a point at the selection center
    **************************************************************************/
    fn show_center_point =
    (
        local point_size = (distance selection.max selection.min) * 0.05
        local current_helper = this.get_point_helper()
        
        current_helper.pos = selection.center
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_Center_"
        
        append this.point_helpers current_helper
    ), --end fn
    
    /**************************************************************************
    Creates and stores a points at the selection min and max
    **************************************************************************/
    fn show_bound_points =
    (
        local point_size = (distance selection.max selection.min) * 0.05
        local current_helper = this.get_point_helper()
        current_helper.pos = selection.min
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_MinXYZ_"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = [selection.min.x,selection.min.y,selection.max.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_MinXY_"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = [selection.max.x,selection.min.y,selection.max.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_MinY_"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = [selection.max.x,selection.min.y,selection.min.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_MinYZ_"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = selection.max
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_MaxXYZ_"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = [selection.max.x,selection.max.y,selection.min.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_MaxXY_"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = [selection.min.x,selection.max.y,selection.min.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_MaxY_"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = [selection.min.x,selection.max.y,selection.max.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_MaxYZ_"
        
        append this.point_helpers current_helper
    ), --end fn
    
    /**************************************************************************
    Creates and stores a points at the averages of the selection
    **************************************************************************/
    fn show_average_points =
    (
        local point_size = (distance selection.max selection.min) * 0.05
        local averages = point3 0 0 0
        
        averages.x = (selection.min.x + selection.max.x) / 2.0
        averages.y = (selection.min.y + selection.max.y) / 2.0
        averages.z = (selection.min.z + selection.max.z) / 2.0
        
        local current_helper = this.get_point_helper()
        current_helper.pos = [averages.x,selection.min.y,averages.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_Average_MinY"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = [selection.max.x,averages.y,averages.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_Average_MaxX"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = [averages.x,selection.max.y,averages.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_Average_MaxY"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = [selection.min.x,averages.y,averages.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_Average_MinX"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = [averages.x,averages.y,selection.max.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_Average_MaxZ"
        
        append this.point_helpers current_helper
        
        current_helper = this.get_point_helper()
        current_helper.pos = [averages.x,averages.y,selection.min.z]
        current_helper.size = point_size
        current_helper.name = uniqueName "Point_Average_MinZ"
        
        append this.point_helpers current_helper
    ), --end fn
    
    /**************************************************************************
    Creates the user requested points
    **************************************************************************/
    fn create_points center_point bound_points average_points size_multiplier =
    (
        --CRITICAL! Clear previous data!
        free this.point_helpers
        
        if center_point do
            this.show_center_point()
        
        if bound_points do
            this.show_bound_points()
        
        if average_points do
            this.show_average_points()
        
        for current in this.point_helpers do
            current.size *= size_multiplier
    ), --end fn
    
    /**************************************************************************
    Returns the position 'point_position' as a color
    **************************************************************************/
    fn get_world_position_color point_position =
    (
        local position_color = color 0 0 0
        
        if point_position.x > 0 do
            position_color.r = 255
        
        if point_position.y > 0 do
            position_color.g = 255
        
        if point_position.z > 0 do
            position_color.b = 255
        
        position_color --return value        
    ), --end fn
    
    /**************************************************************************
    Colors the helpers based on the mode
    **************************************************************************/
    fn color_points color_mode user_color =
    (
        if color_mode == 1 then
        (
            for current in this.point_helpers do
                current.wirecolor = user_color
        ) --end if
        else
        (
            for current in this.point_helpers do
                current.wirecolor = this.get_world_position_color current.pos
        ) --end else
    ) --end fn
) --end struct


( --local scope
	global show_bounds_rollout
    local show_bounds_ini = getDir #plugcfg + "\\LockTransforms.ini"
    local show_bounds_position
	
    /**************************************************************************
    Enables/disables the show bounds button
    **************************************************************************/
    fn enable_button =
    (
        button_state = false
        
        if show_bounds_rollout.center_checkbox.checked or show_bounds_rollout.bounds_checkbox.checked or show_bounds_rollout.average_checkbox.checked do
            button_state = true
        
        show_bounds_rollout.show_bounds_button.enabled = button_state
    ) --end fn

    /**************************************************************************
    UI creation
    **************************************************************************/
	rollout show_bounds_rollout "Show Bounds"
	(
        local bounds = ShowBounds()
        
        groupbox display_group "Display" pos:[5,5] width:140 height:80
            checkbox center_checkbox "Center Point" pos:[15,25] width:100 height:15
            checkbox bounds_checkbox "Min/Max Bounds" pos:[15,45] width:100 height:15 checked:true
            checkbox average_checkbox "Min/Max Averages" pos:[15,65] width:120 height:15
		
        groupbox color_group "Color" pos:[5,90] width:140 height:55
            radioButtons color_radio "" pos:[15,110] labels:#("Constant", "World Position") default:1 columns:1
            colorPicker wirecolor_color "" pos:[100,110] width:35 height:15 color:(color 0 255 128)
        
        groupbox size_group "Size" pos:[5,150] width:140 height:40
            label size_label "Multiplier:" pos:[15,170] width:75 height:15
            spinner size_spinner "" pos:[90,170] width:45 height:16 range:[0.1,100,1] type:#float scale:1.0
            
		button show_bounds_button "Show Bounds" pos:[5,200] width:140 height:20
        label status_label "INFO: Standing by..." pos:[5,225] width:140 height:15
        
        /**********************************************************************
        UI move event handler. Stores the curent position of the UI
        **********************************************************************/
        on show_bounds_rollout moved pos do
            setIniSetting show_bounds_ini "Dialog" "Position" (pos as string)
        
        /**********************************************************************
        Center checkbox event handler. Enables/disables the bounds button
        **********************************************************************/
        on center_checkbox changed the_state do
            enable_button()
        
        /**********************************************************************
        Bound point checkbox event handler. Enables/disables the bounds button
        **********************************************************************/
        on bounds_checkbox changed the_state do
            enable_button()
        
        /**********************************************************************
        Min/max center checkbox event handler. Enables/disables the bounds button
        **********************************************************************/
        on average_checkbox changed the_state do
            enable_button()
        
        /**********************************************************************
        Show bounds event handler. Creates point helpers at the desired bounds
        **********************************************************************/
		on show_bounds_button pressed do
		(
            if selection.count == 0 do
            (
                status_label.text = "ERROR: Nothing selected"
                return false
            ) --end if
            
            with redraw off
            (
                bounds.create_points center_checkbox.checked bounds_checkbox.checked average_checkbox.checked size_spinner.value
                bounds.color_points color_radio.state wirecolor_color.color
            ) --end with
            
            status_label.text = "INFO: Operation complete"
		) --end on
	) --end rollout
    
    show_bounds_position = execute (getIniSetting show_bounds_ini "Dialog" "Position")
    if show_bounds_position == OK do show_bounds_position = [100,100]
    if show_bounds_position.x > sysinfo.desktopsize.x - 150 do
        show_bounds_position.x = sysinfo.desktopsize.x - 155
    if show_bounds_position.y > sysinfo.desktopsize.y - 300 do
        show_bounds_position.y = sysinfo.desktopsize.y - 300
    if show_bounds_position.x < 0 do show_bounds_position.x = 0 
    if show_bounds_position.y < 0 do show_bounds_position.y = 0
    createDialog show_bounds_rollout 150 240 show_bounds_position.x show_bounds_position.y
) --end local scope
